# ===============================================
# Fonction:
# Renvoyer les diffrences sup.  un certain seuil
# entre fichiers de rsultats
# portant sur les mmes couples de polygones.
# Restriction:
# Fonctionne sur des rsultats de cubpack ou de grid,
# mais, pour ces derniers, uniquement s'ils ont t crits avec
# output_format !=1 et != 2.
# On peut comparer des rsultats de cubpack  ceux de grid,
# si ces derniers ont t crits avec output_format !=1 et != 2.
# Arguments:
# fileres1, fileres2: le nom des 2 fichiers,
# ival: le nos des colonnes  comparer, i.e l'indice  partir de 1
#       des valeurs lues sur le fichier
# mdiff : vecteur de longueur gale  ival
#      Lorsque les diffrences en valeur absolue des i-ime valeurs
#      lues sont sup. ou gales  mdiff[i], on les affiche,
#      pour i=1, length(ival)
# datafile: si donn, trac des couples  pbe
# Dans ce cas, il faut aussi donner 'format'
# et igr, vecteur des indices de valeurs dont on veut le
# trac des diff. et plotfile, la racine des postscripts crs
# Retour: une liste qui comprend:
# call: le match.call
# autant de composants v<i>, que de valeurs dans ival,
# <i> dsignant une valeur  de ival:
#     chacun est une matrice dont les colonnes sont les nos
#     des 2 polys, les valeurs lues sur les 2 fichiers et leur
#     diffrence
# Eventuellement:
# Si on constate que, sur les fichiers, les couples tudis
# ne sont pas les mmes, on renvoie 'res1' et 'res2', i.e
# les rsultats lus sous forme de matrice.
# Explications des plots:
# cat("Trace des couples pour lesquels la difference en valeur absolue des \n")
# cat(paste(laf, "imes valeurs est au-dela de ", mdiff[laf], ".\n", sep=""))
# cat("Le poly pour lequel il y a un pbe est en rose,\n")
# cat("tous ses partenaires sont en bleu.\n")

# ===============================================
# Exemple:
# a=crdiff("../examples/ex-data/cub.res",  "../examples/ex-data/grid.res")
# ===================================================

crdiff = function (fileres1, fileres2,
  ival=c(3,4, 10, 11, 15, 16),
  mdiff=c(1.0e-4,1.0e-4, 10, 1, 10,1), 
  datafile=NULL, format=1, delim=" ",
  plotfile="diff",
  igr=c(3,4), ...)
  {
    
    # --------------------------------------------
    # lire les fichiers
# --------------------------------------------
  # Dterminer le nbre de fonctions pour info
# ###     a=scan(fileres1, nmax=1, what=list(rep("character",8)), quiet=T)
# ###     nf1 = as.numeric(a[[1]][6])
# ### cat(nf1, "functions on ", fileres1,"\n")
# ###     
# ###     a=scan(fileres2,nmax=1,  what=list(rep("character",8)), quiet=T)
# ###     nf2 = as.numeric(a[[1]][6])
# ### cat(nf2, "functions on ", fileres2,"\n")
# ### 
# ### #    nfunc= min(nf1,nf2)

  res1 = read.table(fileres1, header=FALSE,skip=1)
  res2=read.table(fileres2,header=FALSE, skip=1)

  dimnames(res1)[[2]]=
    c("p1", "p2",  rep("", (ncol(res1)-2)))
  dimnames(res2)[[2]]=
    c("p1", "p2",  rep("", (ncol(res2)-2)))

# --------------------------------------------
  # Vrifier qu'il s'agit des mmes couples de polys:
# --------------------------------------------
 if ( (nrow(res1) != nrow(res2)) ||
     ! (all(res1[,1]== res2[,1]) && all(res1[,2]==res2[,2]))) {
  warn="The pairs of polygons should be the same on the two files\n"
  retour = list(call=match.call(),
      res1=res1, res2=res2)
    warning(warn)
   return(retour)
  }
    
# --------------------------------------------
    # verifier les argu
# --------------------------------------------
    if ( length(ival) != length(mdiff)) 
      stop("ival and mdiff should have same length\n")
      
# --------------------------------------------
  # Prparer le retour
# --------------------------------------------
  retour = list(call=match.call())
# --------------------------------------------

#-------------------------
# Lire les polygones si trace des couples  pbe:
#-------------------------
      if (!is.null(datafile)) {
        if (format==0)
          stop("No format given\n")

if (format==1)
donnees = readpoly1(filename = datafile, delim=delim)
else
  donnees = readpoly2(filename = datafile, delim=delim)
    
      } # fin datafile
      
# --------------------------------------------
# Calcul des diffrences absolues
# --------------------------------------------
    for (laf in 1:length(ival)) {
      libelle= ival[laf]
      if (libelle > ncol(res1)) {
        cat("No ", libelle, "-st value on file", fileres1,":")
        cat("Ignored\n")
        next
      }
      if (libelle > ncol(res2)) {
        cat("No ", libelle, "-st value on file", fileres2,":")
        cat("Ignored\n")
        next
      }

      diffabs = res1[,libelle]-res2[,libelle]
      lesdiff = (abs(diffabs) >= mdiff[laf])
if (all(lesdiff==FALSE)) {
  mindiff = abs(min(diffabs))
  warning(paste("No absolute difference greater than ",
          mdiff[laf], " for ",
                laf, "-st values\nMinimum difference: ",
                mindiff, "\n", sep=""))
  f1.abs=NULL
}
  else {
f1.abs = matrix( c(res1[lesdiff, "p1"], res1[lesdiff, "p2"],
  res1[lesdiff,libelle], res2[lesdiff,libelle],
  diffabs[lesdiff]),  ncol=5)
  dimnames(f1.abs) = list(NULL,
            c("p1", "p2", "res1",
              "res2","diff"))
}
  
retour[[paste("v",ival[laf], sep="")]] =f1.abs

# --------------------------------------------
      # Trac des couples a pbe
# --------------------------------------------

      if (!is.null(datafile) && (any(igr==ival[laf]))) {
if (is.null(f1.abs)) {
  cat("Nothing to plot\n")
}
else {
z=unique(f1.abs[,2])
nomplot = paste(plotfile,ival[laf],".ps",sep="")
postscript(nomplot)

for (zi in z) {
# Tous les partenaires de zi
    autre = f1.abs[f1.abs[,2]==zi,1]
    dd= donnees
    attr(dd[[as.character(zi)]],"couleur") = "pink"
    for (una in autre) {attr(dd[[as.character(una)]] ,"couleur") = "blue"}
    plot(dd, couleur=FALSE, ...)
# dev.copy2eps(file=nomplot)
  } # fin zi
cat("Create the file ", nomplot, "\n")
dev.off()

}


} # fin datafile
} # fin laf      

         
  return(retour)
} # fin crdiff



