/*--------------- COPYRIGHT ------------------------
| INRA - Unit MIA de Jouy en Josas                |
--------------------------------------------------*/

/*--------------- IDENTIFICATION PRODUIT -----------
| Derniere mise a jour : 15 Fev 2006               |
| Role                 : Calcul de la surface de   |
|   l'intersection de polygones lus sur un fichier |
--------------------------------------------------*/
///////////////////////////////////////////////////////
#include <iostream>

using std::cerr;
using std::endl;

#include "caliinclc.h"
#include "caliconfig.h"
#include "calierror.h"
#include "calimacros.h"
#include "readPoly.h"
#include "util.h"
#include "intersection.h"
#include "geom.h"


 //////////////////////////////////////////////////////////
// Liberer la memoire
//////////////////////////////////////////////////////////
void
libMem (int npoly,
	real ** areainter,
	int *a, int *numPoly, real * area,
	int **ni, char **nomPoly, tPolygoni ** Poly, real ** bary)
{
  DETRU_T2 (areainter, npoly);
  libMemPoly (npoly, a, numPoly, area, ni, nomPoly, Poly, bary);
}				// fin libMem

///////////////////////////////////////////////////////
int
aireintersect (char *filenamei, char *filenamer, Boolean pverbose)
{

  // Les polygones:
  int npoly;



// Va de travail
  int i, j, ii, jj, erreur;
  char lu[MAX_LINE_POLY];
  FILE *fres = NULL, *fpi = NULL;

  char moi[] = "surface";
/* Pour stocker les messages */
  char errmess[CHAR_MAX];
  char errlect[]="Error in reading";

  // On ne le demande pas les paramtres suivants:
  // on leur donne les valeurs par dfaut:

  int pinput = DEFAULT_INPUT_FORMAT;
  char pdelim[] = DEFAULT_DELIM;

//////////////////////////////////////////////////////////
  // Lecture du fichier de polygones
//////////////////////////////////////////////////////////
  fpi = fopen (filenamei, "r");
  if (!fpi)
    {
      sprintf (errmess, "cannot open polygons file %s\n", filenamei);
      return (ecrmess (CALI_ERFIC1, moi, errmess));
    }

  // Lire avec fgets ce fichier, car il a un nbre va. de donnes
  // sur chaque ligne et il ne faut pas mlanger les fscanf et fgets
  if (fgets (lu, MAX_LINE_POLY, fpi) ==NULL)
    return (ecrmess (CALI_ERLECT, moi, errlect, True));
  npoly = atoi (lu);
  //  printf ("\nNumber of polygons: %d\n-------------------\n", npoly);


//////////////////////////////////////////////////////////
  // Declaration des structures de dim npoly:
//////////////////////////////////////////////////////////
  int *a;
  real *area;
  real **areainter;
  int **ni;
  tPolygoni **Poly;
  int *numPoly;			/* les nos d'identification des polys */
  char **nomPoly;		/* les noms des polys */
  real **bary;			/* Les barycentres des polys (inutilis) */

//////////////////////////////////////////////////////////
  // Allocation des structures de dim npoly:
// memory allocation of the structures of dimension=npoly
///////////////////////////////////////////////////////
  CREER_T1 (a, npoly, int);
  CREER_T1 (numPoly, npoly, int);
  CREER_T1 (area, npoly, real);
  CREER_T2 (ni, npoly, int);
  CREER_T2 (nomPoly, npoly, char);
  CREER_T2 (Poly, npoly, tPolygoni);
  CREER_T2 (bary, npoly, real);
  CREER_T2 (areainter, npoly, real);

  for (i = 0; i < npoly; i++)
    {
      CREER_T1 (ni[i], MAX_TRIANGLES, int);
      CREER_T1 (nomPoly[i], MAX_NAME, char);
      CREER_T1 (Poly[i], MAX_TRIANGLES, tPolygoni);
      CREER_T1 (bary[i], DIM, real);
      CREER_T1 (areainter[i], npoly, real);
    }



//////////////////////////////////////////////////////////
// Lecture des polygones
//////////////////////////////////////////////////////////
  tPolygond **Polyd = NULL;	// on ne veut pas les polys corrigs
  int npolybons;
  erreur = ReadPoly (fpi, pverbose, False, pinput, pdelim,
		     npoly, nomPoly, numPoly, npolybons,
		     a, area, ni, Poly, Polyd, bary);

  if ((erreur != OK) && (erreur != CALI_WARNPOLY))
    {
      libMem (npoly, areainter, a, numPoly, area, ni, nomPoly, Poly, bary);
      return (erreur);		// fatal error
    }


  if (npolybons == 0)
    {
      libMem (npoly, areainter, a, numPoly, area, ni, nomPoly, Poly, bary);
      sprintf (errmess, "All polygons are invalid\n");
      return (ecrmess (CALI_ERPOLY7, moi, errmess));
    }


//////////////////////////////////////////////////////////
// Calcul par sous-polys
//////////////////////////////////////////////////////////
  tPolygond D;
  Boolean z;
  tdVertex pointer, ipointer;
  int c;
  tdVertex intersection;
  real aire;


  /* Initialisation du fichier de rsultats */
  if (filenamer != NULL)
    {

      fres = fopen (filenamer, "w");
      if (!fres)
	{
	  libMem (npoly, areainter, a, numPoly, area, ni, nomPoly, Poly,
		  bary);

	  sprintf (errmess, "cannot open file %s\n", filenamer);
	  return (ecrmess (CALI_ERFIC1, moi, errmess));
	}

      // Ecrire le nbre de polys et le nom du fichier d'entree
      fprintf (fres, "npoly:\t%d\tinput-file:\t%s\n", npolybons, filenamei);


    }

  Boolean yai = False;		// true s'il y a des intersections non vides
  /* Boucle sur les couples de polys */
  for (i = 0; i < npoly; i++)
    {
      if (numPoly[i] < 0)
	{
	  // passer au poly suivant
	  continue;
	}

      for (j = i + 1; j < npoly; j++)
	{
	  if (numPoly[j] < 0)
	    {
	      // passer au poly suivant
	      continue;
	    }
	  areainter[i][j] = 0.0;
	  for (ii = 0; ii < a[i]; ii++)
	    {
	      // Mettre le 2ieme sous-poly en rel anti-clockwise
	      for (jj = 0; jj < a[j]; jj++)
		{
		  for (c = 0; c < ni[j][jj]; c++)
		    {
		      D[c][XX] = Poly[j][jj][c][XX];
		      D[c][YY] = Poly[j][jj][c][YY];
		    }

		  NEW (intersection, tdsVertex);
		  intersection->next = intersection->prev = intersection;
		  //  Initialisation du 1ier sommet
		  intersection->v[XX] = intersection->v[YY] = 0;


		  //AB: 18/06/2007
		  // Regarder si un des poly de l'intersection est inclus

		  // dans l'autre car ConvexIntersect ne dtecte pas 
		  // ce cas
		  z = ConvexInclus (Poly[i][ii], ni[i][ii], D, ni[j][jj],
				    intersection);

		  if (z == False)
		    {
		      // Si ce n'est pas le cas:
		      // calculer l'intersection eventuelle

		      z =
			ConvexIntersect (Poly[i][ii], ni[i][ii], D, ni[j][jj],
					 intersection);
		    }


		  if (z == True)
		    {
		      aire = (polygon_area_2 (intersection) / 2);

		      areainter[i][j] += aire;
		      pointer = intersection->next;
		      do
			{
			  ipointer = pointer;
			  pointer = ipointer->next;
			  FREE (ipointer);
			}
		      while (pointer != intersection);
		    }

		  FREE (intersection);
		}		// fin jj
	    }			// fin ii


	  /* Ecrire */
	  if (areainter[i][j] > 0)
	    {
	      yai = True;

	      // Revenir au mtre: 
	      areainter[i][j] = areainter[i][j] / (SCALE * SCALE);
	      if (fres)
		fprintf (fres, "%d\t%d\t%g\n", numPoly[i], numPoly[j],
			 areainter[i][j]);
	      else
		{
		  // crire sur stdout
		  printf ("Idents:\t%d,%d", numPoly[i], numPoly[j]);
		  if (nomPoly[i] != "")
		    printf ("\tnames:\t%s,%s", nomPoly[i], nomPoly[j]);
		  printf ("\tarea:\t%g\n", areainter[i][j]);
		}		// fin else fres
	    }			// fin aire>0

	}			// fin j
    }				// fin i

  if (fres)
    printf ("\nCreated file: %s\n", filenamer);
  else
    {
      if (yai == False)
	printf ("All intersections are null.\n");
    }
  ///////////////////////////////////////
  // Librer la mmoire
  ///////////////////////////////////////

  libMem (npoly, areainter, a, numPoly, area, ni, nomPoly, Poly, bary);

  ///////////////////////////////////////
  // Fermer les fichiers
  ///////////////////////////////////////
  // Pas besoin de fermer fpi: readPoly l'a fait
  if (fres != NULL)
    fclose (fres);

  ///////////////////////////////////////
  // Retour
  ///////////////////////////////////////
  return OK;
}				// fin aireintersect

///////////////////////////////////////////////////////
// Pour crire un message en cas de mauvaise utilisation 
// de la commande ou help
///////////////////////////////////////////////////////
int
errmain (char c, char *cde)
{
/* Pour stocker les messages */
  char errmess[CHAR_MAX];
  char bad[]="bad use";

  if (c != 'h')
    {
      // cas d'erreur:
      sprintf (errmess,
	       "Bad option %c\nUsage:\n %s -h\nor\n %s -i poly-filename  [-r result-filename]\n",
	       c, cde, cde);
      return (ecrmess (CALI_ERSYNTAXE, bad, errmess));
    }
  else
    {
      // cas o on veut l'aide
      printf ("Usage: %s -i poly-filename [-v]  [-r result-filename]\n", cde);
      return (OK);
    }
}				// fin errmain

///////////////////////////////////////////////////////
// Programme principal
///////////////////////////////////////////////////////
int
main (int argc, char *argv[])
{
  char *pfilenamei, *pfilenamer = NULL;
  char c;
  int iarg;
  int narg = argc - 1;

  // True si on veut l'ip des coordonnes lues
  // et de l'indication si les poly sont convexes ou non  
  Boolean pverbose = Boolean (DEFAULT_VERBOSE);


  // Vrifier le nombre d'arguments
  if ((narg < 1) || (narg > 5))
    return (errmain ('h', argv[0]));


  // Dcoder les arguments:
  iarg = 1;
  while (iarg <= narg)
    {
      if (argv[iarg][0] == '-')
	{
	  c = argv[iarg][1];
	  switch (c)
	    {
	    case 'h':
	      return (errmain (c, argv[0]));
	    case 'v':
	      pverbose = True;
	      iarg = iarg - 1;
	      break;
	    case 'i':
	      pfilenamei = argv[iarg + 1];
	      break;
	    case 'r':
	      pfilenamer = argv[iarg + 1];
	      break;
	    default:
	      return (errmain (c, argv[0]));
	      break;
	    }			// fin switch (c)
	  iarg = iarg + 2;
	}			// fin if (argv[iarg][0]== '-')
      else
	{
	  return (errmain (c, argv[0]));
	}
    }				// fin while


  return (aireintersect (pfilenamei, pfilenamer, pverbose));

}				// fin main
