/*--------------- COPYRIGHT ------------------------
| INRA - Unit MIA de Jouy en Josas                |
--------------------------------------------------*/

/*--------------- IDENTIFICATION PRODUIT -----------
| Derniere mise a jour : 15 Fev 2006               |
| Role                 : gre les calculs          |
|    Non interactive version                       |
|    Input arguments:                              |
| pathnames of the polygons-file,                  |
|              the parameters-file,                |
|              the result-file                     |
|    Return: OK or a negative error code.          |
--------------------------------------------------*/
///////////////////////////////////////////////////////
int califlopp_sd (char *filenamei, char *fileparam, char *filenamer);

///////////////////////////////////////////////////////
// Version sans dialogue
///////////////////////////////////////////////////////

#include "caliinclc.h"
#include "calitypes.h"
#include "caliconfig.h"
#include "caliboolean.h"
#include "calierror.h"
#include "calimacros.h"
#include "go.h"
#include "methodIntegr.h"
#include "readPoly.h"
#include "util.h"

//////////////////////////////////////////////////////////
#define EOL '\n'		/* The end-of-line character on the parameter file */

//////////////////////////////////////////////////////////
// Les paramtres de l'tude
//////////////////////////////////////////////////////////

// Le nombre de mots-cls
#define NPARAM  17
// Le nbre de caractres maximal pour le mot-cl associ  chacun:
#define LGPARAM 12
// Les mots-cl associs aux paramtres:
char libParam[NPARAM][LGPARAM] = {
  "verbose", "input", "delim", "output", "nfunc", "method",
  "seed", "stepx", "stepy", "nr",
  "reler", "abser", "maxpts",
  "poly1", "poly2", "nwant", "ncouples"
};


enum paramv
{ verbose = 1, input, delim, output, nfunc, method,
  seed, stepx, stepy, nr,
  reler, abser, maxpts,
  poly1, poly2, nwant, ncouples
};
//////////////////////////////////////////////////////////
// Liberer la memoire
//////////////////////////////////////////////////////////
void
libMem (int npoly,
	int *emet, int *recoit,
	int *a, int *numPoly, real * area,
	int **ni, char **nomPoly, tPolygoni ** Poly, real ** bary)
{



  DETRU_T1 (emet);
  DETRU_T1 (recoit);
  libMemPoly (npoly, a, numPoly, area, ni, nomPoly, Poly, bary);

}				// fin libMem

//////////////////////////////////////////////////////////
	  // Remplir les nos des poly voulus
// ou bien des valeurs relatives  une fonction
//////////////////////////////////////////////////////////
int
rempSend (FILE * fparam, char separator, char type,
	  int nsend, int *send, real * valr)
{

  int ip;
  char carlu;
  char texte[LGPARAM];
  char moi[] = "rempSend";
/* Pour stocker les messages */
  char errmess[CHAR_MAX];
  char errlect[]="Error in reading";

  for (ip = 0; ip < nsend; ip++)
    {
      if (fscanf (fparam, "%s", texte)<1) {
	    return (ecrmess (CALI_ERLECT, moi, errlect, True));
      }
      if (feof (fparam))
	{
	  sprintf (errmess,
		   "premature end of file; %d wanted values waited\n", nsend);
	  return (ecrmess (CALI_ERPARAM3, moi, errmess));
	}			// fin if (feof(fparam))


      while (texte[0] == separator)
	{
	  // Sauter les commentaires
	  while (!feof (fparam) && ((carlu = fgetc (fparam)) != EOL))
	    {
	    }
	  if (fscanf (fparam, "%s", texte)<1) {
	        return (ecrmess (CALI_ERLECT, moi, errlect, True));
	  }
	  if (feof (fparam))
	    {
	      sprintf (errmess,
		       "premature end of file; %d wanted values waited\n",
		       nsend);
	      return (ecrmess (CALI_ERPARAM3, moi, errmess));
	    }			// fin if (feof(fparam))

	}			// fin comment
      if (type == 'i')
	send[ip] = atoi (texte);
      else
	valr[ip] = atof (texte);

    }				// fin ip

  return (OK);
}				// fin rempSend


//////////////////////////////////////////////////////////
	  // Remplir les nos d'un couple poly voulus:
//////////////////////////////////////////////////////////
int
rempCouple (FILE * fparam, int nsend, int *send, int *target)
{

  int ip;
  char carlu;
  char texte[LGPARAM];
  int val2;
  char moi[] = "rempCouple";
/* Pour stocker les messages */
  char errmess[CHAR_MAX];
  char errlect[]="Error in reading";

  for (ip = 0; ip < nsend; ip++)
    {
      if (fscanf (fparam, "%s %d", texte, &val2)<1) {
	    return (ecrmess (CALI_ERLECT, moi, errlect, True));
      }

      if (feof (fparam))
	{
	  sprintf (errmess,
		   "premature end of file; %d wanted polygons waited\n",
		   nsend);
	  return (ecrmess (CALI_ERPARAM3, moi, errmess));
	}			// fin if (feof(fparam))


      while (texte[0] == COMMENT)
	{
	  // Sauter les commentaires
	  while (!feof (fparam) && ((carlu = fgetc (fparam)) != EOL))
	    {
	    }
	  if (fscanf (fparam, "%s %d", texte, &val2)<1) {
	        return (ecrmess (CALI_ERLECT, moi, errlect, True));
	  }
	  if (feof (fparam))
	    {
	      sprintf (errmess,
		       "premature end of file; %d wanted polygons waited\n",
		       nsend);
	      return (ecrmess (CALI_ERPARAM3, moi, errmess));
	    }			// fin if (feof(fparam))

	}			// fin comment
      if (sscanf (texte, "%d", &(send[ip]))<1) {
	return (ecrmess (CALI_ERLECT, moi, errlect, True));
      }
      target[ip] = val2;
    }				// fin ip

  return (OK);
}				// fin rempCouple


//////////////////////////////////////////////////////////
// Renvoyer l'indice  partir de 1 dans le tableau de parametres
// correspondant  un texte donn
//////////////////////////////////////////////////////////
int
getIndexParam (char *texte, char libParam[NPARAM][LGPARAM])
{
  char moi[] = "getIndexParam";
/* Pour stocker les messages */
  char errmess[CHAR_MAX];

  int i;
  for (i = 0; i < NPARAM; i++)
    {
      if (strcmp (texte, libParam[i]) == 0)
	{
	  return (i + 1);
	  break;
	}
    }
  sprintf (errmess, "Unknown keyword:  %s\n", texte);
  return (ecrmess (CALI_ERPARAM1, moi, errmess));
}				// fin getIndexParam




//////////////////////////////////////////////////////////
// Lire un paramtre  sur le fichier des paramtres
// Argument de retour:
// val1Param: la valeur  du paramtre
// Valeur de retour:
//  l'indice du paramtre correspondant
// dans le tableau libParam ou bien
// valeur <0 si erreur ou bien
//  0 si fin de fichier
//////////////////////////////////////////////////////////

int
lit1Param (FILE * fparam,
	   char libParam[NPARAM][LGPARAM], real & val1Param, char &ledelim)
{
  char texte[LGPARAM];
  int i;
  char clu;
  char vallu[LGPARAM];
  char moi[] = "lit1Param";
  char errlect[]="Error in reading";

  if ( fscanf (fparam, "%s", texte)<1) {
    /* fin de fichier = feof (fparam)*/
    return (0);
  }

  while (texte[0] == COMMENT)
    {
      // Sauter les commentaires
      while (!feof (fparam) && ((clu = fgetc (fparam)) != EOL))
	{
	  // ON VA CHERCHER LA LIGNE SUIVANTE
	}
      if (fscanf (fparam, "%s", texte)<1) {
    /* fin de fichier = feof (fparam)*/
    return (0);
  }
    }				// fin WHILE comment
  if (strcmp (texte, "delim") == 0)
    {
      // Cas particulier du dlimiteur de valeurs du fichier
      // des poly: il est entre quotes
      while (!feof (fparam) && ((clu = fgetc (fparam)) != '"'))
	{
	  // On passe au car. qui suit le quote
	}
      if (feof (fparam))
	return (0);
      clu = fgetc (fparam);
      ledelim = clu;
      // Lire le dernier double-quote
      clu = fgetc (fparam);
    }
  else
    if (fscanf (fparam, "%s", vallu)<1) {
      return (0);
  }


  if ((i = getIndexParam (texte, libParam)) < 0)
    return (i);
  val1Param = atof (vallu);

  return (i);
}

// fin lit1Param


///////////////////////////////////////////////////////
// Gestion de l'excution: 
// les paramtres de l'tude sont lus sur un fichier
///////////////////////////////////////////////////////

int
califlopp_sd (char *filenamei, char *fileparam, char *filenamer)
{
//////////////////////////////////////////////////////////
  // Declaration des paramtres:
//////////////////////////////////////////////////////////

  // Paramtres des mthodes
  Boolean grid = Boolean (DEFAULT_METHOD);	// l'indicateur de la mthode voulue
  // Rque: les noms des variables qui contiennent les paramtres
  // sont prfixes par "p" pour les distinguer de l'enum
  // qui contient les textes lus sur le fichier

  // True si on veut l'ip des coordonnes lues
  // et de l'indication si les poly sont convexes ou non  
  Boolean pverbose = Boolean (DEFAULT_VERBOSE);
  // Ce qu'on veut comme impressions  l'cran:
  int poutput = DEFAULT_OUTPUT;

  // Le format du fichier des polygones et le dlimitateur des valeurs
  int pinput = DEFAULT_INPUT_FORMAT;
  char pdelim[2];
  strcpy (pdelim, DEFAULT_DELIM);

  // Grid: dans la version reduite, ces dcl servent
  // uniquement pour la syntaxe.
  real pstepx = real (DEFAULT_STEPX * SCALE);
  real pstepy = real (DEFAULT_STEPY * SCALE);
  int pnr = int (DEFAULT_EST);
  unsigned int pseed = DEFAULT_SEED;


  // Adapt
  int ifunc;
  real pabser[MAX_NFUNCTIONS], preler[MAX_NFUNCTIONS];
  long int pmaxpts[MAX_NFUNCTIONS];

//////////////////////////////////////////////////////////


  // Les polygones:
  int npoly;



  // Les mthodes
  methodIntegr *methode;



// Va de travail
  int i, jt[1], erreur;
  real vt[1];
  char lu[MAX_LINE_POLY];

  // PATH_MAX est defini dans stdio.h
// pathname-prefix of the result-files
  FILE *fparam, *fpi;


  int clu, dlu;
  int nemet = 0;
  int *emet = NULL;
  int *recoit = NULL;


  char moi[] = "califlopp_sd";
/* Pour stocker les messages */
  char errmess[CHAR_MAX];
  char errlect[]="Error in reading";

//////////////////////////////////////////////////////////
  // Lecture du fichier de polygones
//////////////////////////////////////////////////////////

  fpi = fopen (filenamei, "r");
  if (!fpi)
    {
      sprintf (errmess, "cannot open polygons file %s\n", filenamei);
      return (ecrmess (CALI_ERFIC1, moi, errmess));
    }

  // Lire avec fgets ce fichier, car il a un nbre va. de donnes
  // sur chaque ligne et il ne faut pas mlanger les fscanf et fgets
  if (fgets (lu, MAX_LINE_POLY, fpi)==NULL)
    return (ecrmess (CALI_ERLECT, moi, errlect, True));
  npoly = atoi (lu);

//////////////////////////////////////////////////////////
  // Declaration des structures de dim npoly:
//////////////////////////////////////////////////////////
  int *a;
  real *area;
  int **ni;
  tPolygoni **Poly;
  int *numPoly;			/* les nos d'identification des polygons */
  char **nomPoly;		/* les noms des polygons */
  real **bary;			/* Les barycentres des polygons */


//////////////////////////////////////////////////////////
  // Allocation des structures de dim npoly:
// memory allocation of the structures of dimension=npoly
///////////////////////////////////////////////////////
  CREER_T1 (a, npoly, int);
  CREER_T1 (numPoly, npoly, int);
  CREER_T1 (area, npoly, real);
  CREER_T2 (ni, npoly, int);
  CREER_T2 (nomPoly, npoly, char);
  CREER_T2 (Poly, npoly, tPolygoni);
  CREER_T2 (bary, npoly, real);

  for (i = 0; i < npoly; i++)
    {
      CREER_T1 (ni[i], MAX_TRIANGLES, int);
      CREER_T1 (nomPoly[i], MAX_NAME, char);
      CREER_T1 (Poly[i], MAX_TRIANGLES, tPolygoni);
      CREER_T1 (bary[i], DIM, real);
    }


//////////////////////////////////////////////////////////
  // Lecture du fichier de paramtres
//////////////////////////////////////////////////////////
  real valp;
  char carlu;
  int cas = 2;
  int nfunct = DEFAULT_NFUNCTIONS;
  int ifunct[MAX_NFUNCTIONS];	// index from 1 of the dispersion functions
  for (ifunc = 0; ifunc < MAX_NFUNCTIONS; ifunc++)
    {
      ifunct[ifunc] = (ifunc + 1);
      // Cubature parametres
      pabser[ifunc] = DEFAULT_ABS_ERR;
      preler[ifunc] = DEFAULT_REL_ERR;
      pmaxpts[ifunc] = 0;

    }


  if (fileparam != NULL)
    {

      fparam = fopen (fileparam, "r");
      if (!fparam)
	{
	  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly, Poly,
		  bary);
	  sprintf (errmess, "cannot open parameter file %s\n", fileparam);

	  return (ecrmess (CALI_ERFIC1, moi, errmess));
	}


      while (1)
	{
	  i = lit1Param (fparam, libParam, valp, carlu);
	  if (i < 0)
	    {
	      libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
		      Poly, bary);
	      return (i);	// erreur
	    }
	  if (i == 0)
	    break;		// fin de fichier

	  switch (paramv (i))
	    {
	    case verbose:
	      pverbose = Boolean (valp);
	      break;
	    case input:
	      pinput = int (valp);
	      break;
	    case delim:
	      pdelim[0] = carlu;
	      break;
	    case output:
	      poutput = int (valp);
	      if ((poutput < 0) || (poutput > MAX_VAL_OUTPUT))
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);
		  sprintf (errmess,
			   "parameter output %d should be in [0, %d]\n",
			   poutput, MAX_VAL_OUTPUT);
		  return (ecrmess (CALI_ERDIAG7, moi, errmess));
		}

	      break;
	    case nfunc:
	      nfunct = int (valp);
	      if ((nfunct <= 0) || (nfunct > MAX_NFUNCTIONS))
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);
		  sprintf (errmess,
			   "number of function %d should be in [1, %d]\n",
			   ifunc, MAX_NFUNCTIONS);
		  return (ecrmess (CALI_ERDIAG4, moi, errmess));
		}

	      // Lecture des indices des fonctions de dispersion
	      // sauter les commentaires
	      i = rempSend (fparam, '#', 'i', 1, jt, vt);
	      if (i < 0)
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);
		  return (i);	// erreur
		}
	      ifunct[0] = jt[0];
	      // Lire les  indices des fonctions suivants
	      for (ifunc = 1; ifunc < nfunct; ifunc++) {
		if (fscanf (fparam, "%d", &(ifunct[ifunc]))<1) {
		  return (ecrmess (CALI_ERLECT, moi, errlect, True));
		}
	      }
	      break;
	    case reler:
	      // reler est suivi du n0 de la fonction
	      // et sur la ligne suivante, de la precision relative
	      // requise pour cette fonction 
	      ifunc = int (valp);
	      if ((ifunc <= 0) || (ifunc > MAX_NFUNCTIONS))
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);

		  sprintf (errmess,
			   "number of function after reler is %d, should be in [1, %d]\n",
			   ifunc, MAX_NFUNCTIONS);
		  return (ecrmess (CALI_ERDIAG4, moi, errmess));
		}
	      // Sauter les commentaires
	      i = rempSend (fparam, '#', 'r', 1, jt, vt);
	      if (i < 0)
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);
		  return (i);	// erreur
		}
	      preler[ifunc - 1] = vt[0];
	      break;
	    case abser:
	      // abser est suivi du n0 de la fonction
	      // et sur la ligne suivante, de la precision absolue
	      // requise pour cette fonction 
	      ifunc = int (valp);
	      if ((ifunc <= 0) || (ifunc > MAX_NFUNCTIONS))
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);
		  sprintf (errmess,
			   "number of function after abser is %d, should be in [1, %d]\n",
			   ifunc, MAX_NFUNCTIONS);
		  return (ecrmess (CALI_ERDIAG4, moi, errmess));
		}
	      // Sauter les commentaires
	      i = rempSend (fparam, '#', 'r', 1, jt, vt);
	      if (i < 0)
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);
		  return (i);	// erreur
		}
	      pabser[ifunc - 1] = vt[0];
	      break;

	    case maxpts:
	      // maxpts est suivi du n0 de la fonction
	      // et sur la ligne suivante, du nbre max de points
	      // d'valuation pour cette fonction 
	      ifunc = int (valp);
	      if ((ifunc <= 0) || (ifunc > MAX_NFUNCTIONS))
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);

		  sprintf (errmess,
			   "number of function after maxpts is %d, should be in [1, %d]\n",
			   ifunc, MAX_NFUNCTIONS);
		  return (ecrmess (CALI_ERDIAG4, moi, errmess));
		}
	      // Sauter les commentaires
	      i = rempSend (fparam, '#', 'i', 1, jt, vt);
	      if (i < 0)
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);
		  return (i);	// erreur
		}
	      pmaxpts[ifunc - 1] = long (jt[0]);
	      break;
#ifndef REDUCED
	    case method:
	      grid = Boolean (valp);
	      break;
	    case seed:
	      pseed = int (valp);
	      break;
	    case stepx:
	      pstepx = valp * SCALE;
	      break;
	    case stepy:
	      pstepy = valp * SCALE;
	      break;
	    case nr:
	      pnr = int (valp);
	      break;


	    case poly1:
	      clu = int (valp);
	      cas = 1;
	      break;
	    case poly2:
	      dlu = int (valp);
	      break;
	    case nwant:
	      cas = 3;
	      nemet = int (valp);

	      // Remplir les nos des poly voulus:
	      if (nemet < 1)
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);

		  sprintf (errmess, "number of polygon must be >=1\n");
		  return (ecrmess (CALI_ERDIAG2, moi, errmess));
		}
	      CREER_T1 (emet, nemet, int);
	      if ((erreur = rempSend (fparam, COMMENT, 'i',
				      nemet, emet, vt)) != OK)
		return (erreur);

	      break;


	      // Le cas 4 (ntarget, nsend) est supprim

	    case ncouples:
	      cas = 5;
	      nemet = int (valp);

	      // Remplir les nos des poly metteurs:
	      if (nemet < 1)
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);

		  sprintf (errmess, "number of polygon must be >=1\n");
		  return (ecrmess (CALI_ERDIAG2, moi, errmess));
		}
	      CREER_T1 (emet, nemet, int);
	      CREER_T1 (recoit, nemet, int);
	      if ((erreur = rempCouple (fparam, nemet, emet, recoit)) != OK)
		{
		  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
			  Poly, bary);
		  return (erreur);
		}

	      break;
#endif
	    default:
	      libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly,
		      Poly, bary);
	      sprintf (errmess, "Unknown keyword\n");
	      return (ecrmess (CALI_ERPARAM1, moi, errmess));

	    }			// fin switch
	}			// fin boucle while

#ifndef REDUCED
      if ((cas == 1) && (dlu <= 0))
	{
	  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly, Poly,
		  bary);
	  sprintf (errmess, "when poly1 is set, poly2 should be set\n");
	  return (ecrmess (CALI_ERPARAM2, moi, errmess));
	}
#endif
    }				// fin fileparam

//////////////////////////////////////////////////////////
// Le copyright
//////////////////////////////////////////////////////////
  if ((poutput != NOTHING) && (poutput != LIGHT))
    printf ("CaliFloPP -  Copyright (c) 2007 - INRA\n");


//////////////////////////////////////////////////////////
// Lecture des polygones
//////////////////////////////////////////////////////////
  Boolean calcSurf = False;	// on ne veut pas que les surfaces
  // (calcSurf n'est vrai que pour le programme surface)

  tPolygond **Polyd = NULL;	// on ne veut pas les polys corrigs
  // (Polyd n'est pas NULL que pour le programme polysimplif)

  int npolybons;

  erreur = ReadPoly (fpi, pverbose, calcSurf, pinput, pdelim,
		     npoly, nomPoly, numPoly, npolybons,
		     a, area, ni, Poly, Polyd, bary);


  if ((erreur != OK) && (erreur != CALI_WARNPOLY))
    {
      libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly, Poly, bary);
      return (erreur);		// fatal error
    }

  if (npolybons == 0)
    {
      libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly, Poly, bary);
      sprintf (errmess, "All polygons are invalid\n");
      return (ecrmess (CALI_ERPOLY7, moi, errmess));
    }
  if ((poutput != NOTHING) && (poutput != LIGHT))
    {
      printf ("\nNumber of polygons: %d\n-------------------\n", npoly);
    }


  /////////////////////////////////////////////////////
#ifndef REDUCED
  if ((nemet > 0) && (nemet > npoly) && (cas == 3))
    {
      libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly, Poly, bary);

      sprintf (errmess, "number of wanted polygons %d should be in [1-%d]\n",
	       nemet, npoly);
      return (ecrmess (CALI_ERDIAG2, moi, errmess));
    }
#endif


//////////////////////////////////////////////////////////
// Faire les calculs
//////////////////////////////////////////////////////////
  char openr[]="w";
  if ((erreur =
       suite (cas, pverbose, pinput, poutput, grid,
	      pstepx, pstepy, pnr, pseed,
	      preler, pabser, pmaxpts,
	      nfunct, ifunct,
	      npoly, clu, dlu,
	      nemet, emet, recoit,
	      a, area, bary,
	      ni, Poly,
	      numPoly, nomPoly, filenamei, filenamer, openr, methode)) != OK)
    {
      libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly, Poly, bary);
      return (erreur);
    }

  ///////////////////////////////////////
  // Librer la mmoire
  ///////////////////////////////////////
  libMem (npoly, emet, recoit, a, numPoly, area, ni, nomPoly, Poly, bary);

  ///////////////////////////////////////
  // crire les fichiers crs
  ///////////////////////////////////////


//  if (strncmp(filenamer , " ", 1) !=0) 
  if ((poutput != NOTHING) && (poutput != LIGHT) && (filenamer != NULL))
    // c'est go qui le ferme
    printf ("\nCreated file: %s\n", filenamer);

  return OK;
}

// fin califlopp_sd
