/*--------------- COPYRIGHT ------------------------
| INRA - Unit MIA de Jouy en Josas                |
--------------------------------------------------*/

///////////////////////////////////////////////////////
// La version
///////////////////////////////////////////////////////
#define VERSION "Version 2010-06-30"
// Previous:  "Version 2009-09-22"
// Previous:  "Version 2007-12-13"
// Previous:  "Version 2007-12-05"
// Previous:  "Version 2007-11-12"
// Previous:  "Version 2007-10-23"
// Previous: "Version 2007-08-31"
// Previous: "Version 2007-07-04"
//  Previous: "Version 2007-06-21"


///////////////////////////////////////////////////////
//  includes
///////////////////////////////////////////////////////

#include <iostream>

using std::cerr;
using std::cout;
using std::endl;
#include "caliinclc.h"
#include "calitypes.h"
#include "calierror.h"
#include "caliconfig.h"
#include "util.h"

///////////////////////////////////////////////////////
// programmes externes
///////////////////////////////////////////////////////
/* interactive programme */
int califlopp_ad ();

/* no-interactive programme */
int califlopp_sd (char *filenamei, char *fileparam, char *filenamer);

///////////////////////////////////////////////////////
// Fonction appele quand exit
// Facultative
// // void
// surexit( void){ }
///////////////////////////////////////////////////////

///////////////////////////////////////////////////////
// Pour crire un message en cas de mauvaise utilisation 
// de la commande ou help
///////////////////////////////////////////////////////
void
errmain (char *cde)
{
  cerr << "Usage: " << endl
    << cde << endl
    << cde << " -h[elp]" << endl
    << cde << " -v[ersion] " << endl
    << cde <<
    " -i polygons-filename [-p parameters-filename] [-r result-filename]" <<
    endl;
}				// fin errmain



///////////////////////////////////////////////////////
// Programme principal
///////////////////////////////////////////////////////
int
main (int argc, char *argv[])
{
  char *pfilenamei = NULL, *pfilenamep = NULL, *pfilenamer = NULL;
  char c;
  int code, iarg;
  int narg = argc - 1;


  // Fonction sur exit: facultative atexit(surexit);
#ifndef REDUCED
  if (narg == 0)
    {
      // version avec dialogue: il n'y a pas d'argument
      if ((code = califlopp_ad ()) != OK)
	{
	  cerr << "Sorry: program CaliFloPP failed! (code " << code <<")" << endl;
	  if (DEFAULT_INPUT_FORMAT !=1) 
	  cerr <<"Interactive mode works only when CaliFloPP has been configured with DEFAULT_INPUT_FORMAT=1 " << endl;

	}
      return (code);
    }
#endif

  // Vrifier le nombre d'arguments
  if ((narg != 1) && (narg != 2) && (narg != 4) && (narg != 6))
    {
      errmain (argv[0]);
      return (CALI_ERSYNTAXE);
    }



  if (narg != 6)
    // on ne veut pas de fichier rsultat
    pfilenamer = NULL;

  // Dcoder les arguments:
  iarg = 1;
  while (iarg <= narg)
    {
      if (argv[iarg][0] == '-')
	{
	  c = argv[iarg][1];
	  switch (c)
	    {
	    case 'h':
	      errmain (argv[0]);
	      return (OK);
	    case 'v':
	      cout << VERSION << endl;
	      return (OK);
	    case 'i':
	      pfilenamei = argv[iarg + 1];
	      break;
	    case 'p':
	      pfilenamep = argv[iarg + 1];
	      break;
	    case 'r':
	      pfilenamer = argv[iarg + 1];
	      break;
	    default:
	      errmain (argv[0]);
	      return (CALI_ERSYNTAXE);
	    }			// fin switch (c)
	  iarg = iarg + 2;
	}			// fin if (argv[iarg][0]== '-')
      else
	{
	  errmain (argv[0]);
	  return (CALI_ERSYNTAXE);
	}
    }				// fin while

  if ((code = califlopp_sd (pfilenamei, pfilenamep, pfilenamer)) != OK)
    {
      cerr << "Sorry: program CaliFloPP failed!" << endl;
    }
  return (code);

}				// fin main
